#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-17-05
#
# Check Point Identity Awareness status
#
# 2018-03-15: code cleanup, added identity counters 39-44
# 2020-06-08: changed snmp-scan function
# 2021-08-25: rewritten for CMK 2.0
# 2023-0529: moved gui files to ~/local/lib/check_mk/gui/plugins/...
#

#
# snmpwalk sample
#
# .1.3.6.1.4.1.2620.1.38.1.0 = STRING: "Identity Awareness"
# .1.3.6.1.4.1.2620.1.38.2.0 = Gauge32: 149
# .1.3.6.1.4.1.2620.1.38.3.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.4.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.5.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.7.0 = Gauge32: 149
# .1.3.6.1.4.1.2620.1.38.8.0 = Gauge32: 188
# .1.3.6.1.4.1.2620.1.38.9.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.10.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.11.0 = Gauge32: 337
# .1.3.6.1.4.1.2620.1.38.12.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.13.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.14.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.15.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.16.0 = Gauge32: 221
# .1.3.6.1.4.1.2620.1.38.17.0 = Gauge32: 44
# .1.3.6.1.4.1.2620.1.38.18.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.19.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.20.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.21.0 = Gauge32: 2272
# .1.3.6.1.4.1.2620.1.38.22.0 = Gauge32: 68786
# .1.3.6.1.4.1.2620.1.38.23.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.39.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.40.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.41.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.42.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.43.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.44.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.101.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.38.102.0 = STRING: "OK"
# .1.3.6.1.4.1.2620.1.38.103.0 = STRING: "OK"
#
# sample info
# [[u'Identity Awareness', u'149', u'0', u'0', u'0', u'0', u'149', u'188', u'0', u'0', u'337', u'0', u'0', u'0', u'0',
#   u'221', u'44', u'0', u'0', u'0', u'2272', u'68786', u'0', u'0', u'0', u'0', u'0', u'0', u'0', u'0', u'OK', u'OK']]
#

import time
from dataclasses import dataclass
from typing import Optional, Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    get_rate,
    GetRateError,
    get_value_store,
    Result,
    State,
    Metric,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointIaStatus:
    iaStatus: int
    iaStatusShortDesc: str
    iaStatusLongDesc: str
    metrics_count: Dict[str, int]
    metrics_rate: Dict[str, int]


def parse_checkpoint_identity_awareness(string_table: StringTable) -> Optional[CheckpointIaStatus]:
    try:
        iaAuthUsers, iaUnAuthUsers, iaAuthUsersKerberos, iaAuthMachKerberos, iaAuthUsersPass, \
        iaAuthUsersADQuery, iaAuthMachADQuery, iaLoggedInAgent, iaLoggedInCaptivePortal, iaLoggedInADQuery, \
        iaAntiSpoffProtection, iaSuccUserLoginKerberos, iaSuccMachLoginKerberos, iaSuccUserLoginPass, \
        iaSuccUserLoginADQuery, iaSuccMachLoginADQuery, iaUnSuccUserLoginKerberos, iaUnSuccMachLoginKerberos, \
        iaUnSuccUserLoginPass, iaSuccUserLDAP, iaUnSuccUserLDAP, iaDataTrans, iaRADIUSAccounting, \
        iaIdentityCollectorActiveDirectory, iaIdentityCollectorCiscoISE, iaTerminalServer, iaRemoteAccess, \
        iaIdentityWebAPI, iaStatus, iaStatusShortDesc, iaStatusLongDesc = string_table[0]
    except IndexError:
        return

    return CheckpointIaStatus(
        iaStatus=int(iaStatus),
        iaStatusShortDesc=iaStatusShortDesc,
        iaStatusLongDesc=iaStatusLongDesc,
        metrics_count={
            'iaAuthUsers': int(iaAuthUsers),
            'iaUnAuthUsers': int(iaUnAuthUsers),
            'iaAuthUsersKerberos': int(iaAuthUsersKerberos),
            'iaAuthMachKerberos': int(iaAuthMachKerberos),
            'iaAuthUsersPass': int(iaAuthUsersPass),
            'iaAuthUsersADQuery': int(iaAuthUsersADQuery),
            'iaAuthMachADQuery': int(iaAuthMachADQuery),
            'iaLoggedInAgent': int(iaLoggedInAgent),
            'iaLoggedInCaptivePortal': int(iaLoggedInCaptivePortal),
            'iaLoggedInADQuery': int(iaLoggedInADQuery),
            'iaAntiSpoffProtection': int(iaAntiSpoffProtection),
            'iaSuccUserLoginKerberos': int(iaSuccUserLoginKerberos),
            'iaSuccMachLoginKerberos': int(iaSuccMachLoginKerberos),
            'iaSuccUserLoginPass': int(iaSuccUserLoginPass),
            'iaSuccUserLoginADQuery': int(iaSuccUserLoginADQuery),
            'iaSuccMachLoginADQuery': int(iaSuccMachLoginADQuery),
            'iaUnSuccUserLoginKerberos': int(iaUnSuccUserLoginKerberos),
            'iaUnSuccMachLoginKerberos': int(iaUnSuccMachLoginKerberos),
            'iaUnSuccUserLoginPass': int(iaUnSuccUserLoginPass),
            'iaDataTrans': int(iaDataTrans),
            'iaRADIUSAccounting': int(iaRADIUSAccounting) if iaRADIUSAccounting.isdigit() else None,
            'iaIdentityCollectorActiveDirectory': int(iaIdentityCollectorActiveDirectory) if iaIdentityCollectorActiveDirectory.isdigit() else None,
            'iaIdentityCollectorCiscoISE': int(iaIdentityCollectorCiscoISE) if iaIdentityCollectorCiscoISE.isdigit() else None,
            'iaTerminalServer': int(iaTerminalServer) if iaTerminalServer.isdigit() else None,
            'iaRemoteAccess': int(iaRemoteAccess) if iaRemoteAccess.isdigit() else None,
            'iaIdentityWebAPI': int(iaIdentityWebAPI) if iaIdentityWebAPI.isdigit() else None,
        },
        metrics_rate={
            'iaSuccUserLDAP': int(iaSuccUserLDAP),
            'iaUnSuccUserLDAP':int(iaUnSuccUserLDAP),
        },
    )


def discovery_checkpoint_identity_awareness(section: CheckpointIaStatus) -> DiscoveryResult:
    yield Service()


def check_checkpoint_identity_awareness(params, section: CheckpointIaStatus) -> CheckResult:

    yield Result(state=State.OK, summary=f'Authenticated users: {section.metrics_count["iaAuthUsers"]}')
    yield Result(state=State.OK, summary=f'Unauthenticated users: {section.metrics_count["iaUnAuthUsers"]}')

    if not section.iaStatus == 0:
        yield Result(state=State.WARN, notice=section.iaStatusLongDesc)

    for key, value in section.metrics_count.items():
        if value is not None and key not in params['ignore_counters']:
            yield Metric(
                value=value,
                name=f'checkpoint_identity_awareness_{key}',
            )

    now_time = time.time()
    value_store = get_value_store()
    for key, value in section.metrics_rate.items():
        if value is not None and key not in params['ignore_counters']:
            try:
                value = get_rate(value_store, f'checkpoint_identity_awareness_{key}', now_time, value, raise_overflow=True)
            except GetRateError:
                value = 0
            yield Metric(
                value=value,
                name=f'checkpoint_identity_awareness_{key}',
            )


register.snmp_section(
    name='checkpoint_identity_awareness',
    parse_function=parse_checkpoint_identity_awareness,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.38',  # CHECKPOINT-MIB::identityAwareness
        oids=[
            '2',  # identityAwarenessAuthUsers
            '3',  # identityAwarenessUnAuthUsers
            '4',  # identityAwarenessAuthUsersKerberos
            '5',  # identityAwarenessAuthMachKerberos
            '6',  # identityAwarenessAuthUsersPass
            '7',  # identityAwarenessAuthUsersADQuery
            '8',  # identityAwarenessAuthMachADQuery
            '9',  # identityAwarenessLoggedInAgent
            '10',  # identityAwarenessLoggedInCaptivePortal
            '11',  # identityAwarenessLoggedInADQuery
            '12',  # identityAwarenessAntiSpoffProtection
            '13',  # identityAwarenessSuccUserLoginKerberos
            '14',  # identityAwarenessSuccMachLoginKerberos
            '15',  # identityAwarenessSuccUserLoginPass
            '16',  # identityAwarenessSuccUserLoginADQuery
            '17',  # identityAwarenessSuccMachLoginADQuery
            '18',  # identityAwarenessUnSuccUserLoginKerberos
            '19',  # identityAwarenessUnSuccMachLoginKerberos
            '20',  # identityAwarenessUnSuccUserLoginPass
            '21',  # identityAwarenessSuccUserLDAP
            '22',  # identityAwarenessUnSuccUserLDAP
            '23',  # identityAwarenessDataTrans

            '39',  # identityAwarenessRADIUSAccounting
            '40',  # identityAwarenessIdentityCollectorActiveDirectory
            '41',  # identityAwarenessIdentityCollectorCiscoISE
            '42',  # identityAwarenessTerminalServer
            '43',  # identityAwarenessRemoteAccess
            '44',  # identityAwarenessIdentityWebAPI

            '101',  # identityAwarenessStatus
            '102',  # identityAwarenessStatusShortDesc
            '103',  # identityAwarenessStatusLongDesc
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_identity_awareness',
    service_name='IA summary',
    discovery_function=discovery_checkpoint_identity_awareness,
    check_function=check_checkpoint_identity_awareness,
    check_default_parameters={
       'ignore_counters': [],
    },
    check_ruleset_name='checkpoint_identity_awareness',
)
